library ieee;
use ieee.std_logic_1164.all;

ENTITY cpu1 IS
	PORT
	(		-- Input ports
			clk		: in	std_logic;
			mem_clk	: in	std_logic;
			rst		: in	std_logic;
			dataIn	: in	std_logic_vector(31 downto 0);
			-- Output ports
			dataOut		: out	std_logic_vector(31 downto 0);
			addrOut		: out	std_logic_vector(31 downto 0);
			wEn 			: out	std_logic;								-- For Instruction Memory
			-- Debug data.
			dOutA, dOutB	: out	std_logic_vector(31 downto 0);
			dOutC, dOutZ	: out	std_logic;
			dOutIR			: out	std_logic_vector(31 downto 0);
			dOutPC			: out	std_logic_vector(31 downto 0);
			outT				: out	std_logic_vector(2 downto 0);
			wen_mem, en_mem : out std_logic							-- For data memory
	);	
END cpu1;

ARCHITECTURE behavior OF cpu1 IS

-------------------------------Internal Components---------------------------------------------------
	COMPONENT reset_circuit IS
		PORT (
			Reset 		: IN STD_LOGIC;
			Clk 			: IN STD_LOGIC;
			Enable_PD 	: OUT STD_LOGIC;
			Clr_PC 		: OUT STD_LOGIC
		);
	END COMPONENT;
	
	COMPONENT control IS
		PORT (
			clk, mclk 						: IN STD_LOGIC;
			enable 							: IN STD_LOGIC;
			statusC, statusZ 				: IN STD_LOGIC;
			INST 								: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
			A_Mux, B_Mux 					: OUT STD_LOGIC;
			IM_MUX1, REG_Mux 				: OUT STD_LOGIC;
			IM_MUX2, DATA_Mux 			: OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			ALU_op 							: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			inc_PC, ld_PC 					: OUT STD_LOGIC;
			clr_IR 							: OUT STD_LOGIC;
			ld_IR 							: OUT STD_LOGIC;
			clr_A, clr_B, clr_C, clr_Z : OUT STD_LOGIC;
			ld_A, ld_B, ld_C, ld_Z 		: OUT STD_LOGIC;
			T 									: OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
			wen, en 							: OUT STD_LOGIC
		);
	END COMPONENT;
	
	COMPONENT datapath IS
		PORT (
			Clk, mClk 						: IN STD_LOGIC; 								-- clock Signal
	
			WEN, EN 							: IN STD_LOGIC;			  						--Memory Signals

			Clr_A , Ld_A 					: IN STD_LOGIC;	  						-- Register Control Signals (CLR and LD).
			Clr_B , Ld_B 					: IN STD_LOGIC;
			Clr_C , Ld_C 					: IN STD_LOGIC;
			Clr_Z , Ld_Z 					: IN STD_LOGIC;
			Clr_PC , Ld_PC 				: IN STD_LOGIC;
			Clr_IR , Ld_IR 				: IN STD_LOGIC;

			Out_A 							: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);		-- Register outputs (Some needed to feed back to control unit. Others pulled out for testing.
			Out_B 							: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
			Out_C 							: OUT STD_LOGIC;
			Out_Z 							: OUT STD_LOGIC;
			Out_PC 							: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
			Out_IR 							: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);

			Inc_PC 							: IN STD_LOGIC;									-- Special inputs to PC.

			ADDR_OUT 						: OUT STD_LOGIC_VECTOR(31 DOWNTO 0); 	-- Address and Data Bus signals for debugging. 
			DATA_IN 							: IN STD_LOGIC_VECTOR(31 DOWNTO 0); 
			DATA_OUT, MEM_OUT, MEM_IN 	: OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
			MEM_ADDR 						: OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 
			DATA_Mux							: IN STD_LOGIC_VECTOR(1 DOWNTO 0); 		-- Various MUX controls.
			REG_Mux 							: IN STD_LOGIC; 
			A_MUX, B_MUX 					: IN STD_LOGIC; 
			IM_MUX1 							: IN STD_LOGIC; 
			IM_MUX2 							: IN STD_LOGIC_VECTOR(1 DOWNTO 0);
			
			
			ALU_Op 							: IN STD_LOGIC_VECTOR(2 DOWNTO 0)
		);
	END COMPONENT;
	
-------------------------------Internal Wires----------------------------------------------------------
	SIGNAL en_PD_sig	:	STD_LOGIC;
	SIGNAL clr_PC_sig	:	STD_LOGIC;
	
	SIGNAL en_sig																	: 	STD_LOGIC;
	SIGNAL statusC_sig, statusZ_sig											: 	STD_LOGIC;
	SIGNAL INST_sig																: 	STD_LOGIC_VECTOR(31 DOWNTO 0);
	SIGNAL A_Mux_sig, B_Mux_sig												: 	STD_LOGIC;
	SIGNAL IM_MUX1_sig, REG_Mux_sig											: 	STD_LOGIC;
	SIGNAL IM_MUX2_sig, DATA_Mux_sig											: 	STD_LOGIC_VECTOR(1 DOWNTO 0);
	SIGNAL ALU_op_sig																: 	STD_LOGIC_VECTOR(2 DOWNTO 0);
	SIGNAL inc_PC_sig, ld_PC_sig												: 	STD_LOGIC;
	SIGNAL clr_IR_sig, clr_A_sig, clr_B_sig, clr_C_sig, clr_Z_sig	:	STD_LOGIC;
	SIGNAL ld_IR_sig, ld_A_sig, ld_B_sig, ld_C_sig, ld_Z_sig			:	STD_LOGIC;
	
	SIGNAL mem_out_sig, mem_in_sig 						: STD_LOGIC_VECTOR(31 DOWNTO 0);
	SIGNAL mem_addr_sig										: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL wen_mem_sig, en_mem_sig						: STD_LOGIC;
	
-------------------------------Component Instantiations-------------------------------------------------
BEGIN
	Reset_Circuit1	:	reset_circuit
	PORT MAP
	(
		Reset 		=> rst,
		Clk 			=>	clk,
		Enable_PD 	=> en_PD_sig,					-- enable signal
		Clr_PC 		=> clr_PC_sig
	);
	
	Control_Circuit	:	control
	PORT MAP
	(
		clk 		=>	clk,
		mclk 		=>	mem_clk,
		enable 	=>	en_PD_sig,						-- enable signal
		statusC 	=> statusC_sig,
		statusZ 	=>	statusZ_sig,			
		INST 		=>	dataIn,						
		A_Mux 	=>	A_Mux_sig,
		B_Mux 	=>	B_Mux_sig,	
		IM_MUX1 	=> IM_MUX1_sig,
		REG_Mux 	=> REG_Mux_sig,
		IM_MUX2	=> IM_MUX2_sig,
		DATA_Mux => DATA_Mux_sig,
		ALU_op 	=> ALU_op_sig,
		inc_PC	=> inc_PC_sig,
		ld_PC 	=> ld_PC_sig,
		clr_IR 	=> clr_IR_sig,
		ld_IR 	=>	ld_IR_sig,
		clr_A		=> clr_A_sig,
		clr_B		=> clr_B_sig,
		clr_C		=> clr_C_sig,
		clr_Z 	=> clr_Z_sig,
		ld_A		=>	ld_A_sig,
		ld_B		=> ld_B_sig,
		ld_C		=>	ld_C_sig,
		ld_Z 		=>	ld_Z_sig,
		T 			=>	outT,
		wen		=>	wen_mem_sig,					
		en 		=> en_mem_sig
	);
	
	Datapath_Circuit	:	datapath
	PORT MAP
	(
		Clk			=>	clk,
		mClk 			=>	mem_clk,	
	
		WEN			=> wen_mem_sig,
		EN 			=>	en_mem_sig,		

		Clr_A			=> clr_A_sig,
		Ld_A 			=>	ld_A_sig,
		Clr_B			=>	clr_B_sig,
		Ld_B 			=>	ld_B_sig,
		Clr_C			=>	clr_C_sig,
		Ld_C 			=>	ld_C_sig,
		Clr_Z			=>	clr_Z_sig,
		Ld_Z 			=>	ld_Z_sig,
		Clr_PC		=>	clr_PC_sig,
		Ld_PC 		=>	ld_PC_sig,
		Clr_IR		=>	clr_IR_sig,
		Ld_IR 		=>	ld_IR_sig,

		Out_A 		=>	dOutA,			
		Out_B 		=>	dOutB,		
		Out_C 		=>	statusC_sig,		
		Out_Z 		=>	statusZ_sig,		
		Out_PC 		=>	dOutPC,		
		Out_IR 		=>	dOutIR,		

		Inc_PC 		=>	inc_PC_sig,				

		ADDR_OUT		=> addrOut,					
		DATA_IN 		=>	dataIn,	
		DATA_OUT		=>	dataOut,
		MEM_OUT		=>	mem_out_sig,
		MEM_IN		=> mem_in_sig,
		MEM_ADDR 	=>	mem_addr_sig,
		 
		DATA_Mux		=>	DatA_Mux_sig,				
		REG_Mux 		=>	REG_Mux_sig,				
		A_MUX			=>	A_Mux_sig,
		B_MUX 		=>	B_Mux_sig,
		IM_MUX1 		=>	IM_MUX1_sig,				 
		IM_MUX2 		=>	IM_MUX2_sig,
		
		ALU_Op 		=>	ALU_op_sig
	);
	
	wen_mem <= wen_mem_sig;
	en_mem  <= en_mem_sig;
	dOutC   <= statusC_sig;
	dOutZ	  <= statusZ_sig;
	
	wEn <= '0';
END behavior;