----------------------------------------------Datapath----------------------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;

ENTITY datapath IS
	PORT( Clk, mClk : IN STD_LOGIC; 						-- clock Signal
	
	WEN, EN : IN STD_LOGIC;			  						--Memory Signals

	Clr_A , Ld_A : IN STD_LOGIC;	  						-- Register Control Signals (CLR and LD).
	Clr_B , Ld_B : IN STD_LOGIC;
	Clr_C , Ld_C : IN STD_LOGIC;
	Clr_Z , Ld_Z : IN STD_LOGIC;
	Clr_PC , Ld_PC : IN STD_LOGIC;
	Clr_IR , Ld_IR : IN STD_LOGIC;

	Out_A : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);		-- Register outputs (Some needed to feed back to control unit. Others pulled out for testing.
	Out_B : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
	Out_C : OUT STD_LOGIC;
	Out_Z : OUT STD_LOGIC;
	Out_PC : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
	Out_IR : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);

	Inc_PC : IN STD_LOGIC;									-- Special inputs to PC.

	ADDR_OUT : OUT STD_LOGIC_VECTOR(31 DOWNTO 0); 	-- Address and Data Bus signals for debugging. 
	DATA_IN : IN STD_LOGIC_VECTOR(31 DOWNTO 0); 
	DATA_OUT, MEM_OUT, MEM_IN : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
	MEM_ADDR : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
 
	DATA_Mux: IN STD_LOGIC_VECTOR(1 DOWNTO 0); 		-- Various MUX controls.
	REG_Mux : IN STD_LOGIC; 
	A_MUX, B_MUX : IN STD_LOGIC; 
	IM_MUX1 : IN STD_LOGIC; 
	IM_MUX2 : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
	
	
	ALU_Op : IN STD_LOGIC_VECTOR(2 DOWNTO 0));		-- ALU Operations.
END datapath;

---------------------------------------------------------------------------------------------------------------------------
ARCHITECTURE description OF datapath IS
	------------------------------COMPONENT INSTANTIATIONS------------------------------
	
	-- RED component
	COMPONENT red IS
		PORT (	
					source	: IN STD_LOGIC_VECTOR (31 downto 0);
					dest		: OUT UNSIGNED (7 downto 0)
			);
	END COMPONENT;
	
	-- UZE component
	COMPONENT uze IS
		PORT (	
					source	: IN STD_LOGIC_VECTOR (31 downto 0);
					dest		: OUT STD_LOGIC_VECTOR (31 downto 0)
			);
	END COMPONENT;
	
	-- LZE component
	COMPONENT lze IS
		PORT (	
					source	: IN STD_LOGIC_VECTOR (31 downto 0);
					dest		: OUT STD_LOGIC_VECTOR (31 downto 0)
			);
	END COMPONENT;
	
	-- 2to1 MUX
	COMPONENT mux_2to1 IS
		PORT (	
					in0,in1	: IN STD_LOGIC_VECTOR(31 downto 0);	--MUX signal input choices										
					sel 		: IN STD_LOGIC;							--Select		(1-bit)							
					y 			: OUT STD_LOGIC_VECTOR(31 downto 0)	--MUX output signal
			);
	END COMPONENT;
	
	-- 4to1 MUX
	COMPONENT mux_4to1 IS
		PORT (		
					in0,in1,in2,in3	: IN STD_LOGIC_VECTOR(31 downto 0);	--MUX signal input choices										
					sel 					: IN STD_LOGIC_VECTOR(1 downto 0);	--Select									
					y 						: OUT STD_LOGIC_VECTOR(31 downto 0)	--MUX output signal	
			);
	END COMPONENT;
	
	-- Program Counter (PC)
	COMPONENT pc IS
		PORT (
					clr 	: IN STD_LOGIC;
					clk 	: IN STD_LOGIC;
					ld 	: IN STD_LOGIC;
					inc 	: IN STD_LOGIC;
					d 		: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
					q 		: INOUT STD_LOGIC_VECTOR(31 DOWNTO 0)
			);
	END COMPONENT;
	
	-- 1-bit Register
	COMPONENT register1 IS
		PORT (
					d 		: IN STD_LOGIC;						--SW[2]			input
					ld 	: IN STD_LOGIC;						--SW[3]			load/enable
					clk 	: IN STD_LOGIC;						--SW[0]			async. clear
					clr 	: IN STD_LOGIC;						--SW[1]			clock
					Q 		: OUT STD_LOGIC						--LEDG[0]		output
			);
	END COMPONENT;
	
	-- 32-bit Register
	COMPONENT register32 IS
		PORT (
					d 		: IN STD_LOGIC_VECTOR(31 DOWNTO 0);				--input
					ld 	: IN STD_LOGIC;										--load/enable
					clk 	: IN STD_LOGIC;										--async. clear
					clr 	: IN STD_LOGIC;										--clock
					Q 		: OUT STD_LOGIC_VECTOR(31 DOWNTO 0)				--output
			);
	END COMPONENT;
	
	-- Data Memory
	COMPONENT data_mem IS
		PORT (
					clk 		: IN STD_LOGIC;							
					addr 		: IN UNSIGNED(7 DOWNTO 0);
					data_in 	: IN STD_LOGIC_VECTOR(31 DOWNTO 0);
					wen 		: IN STD_LOGIC;
					en 		: IN STD_LOGIC;
					data_out : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
			);
	END COMPONENT;
	
	-- ALU component
	COMPONENT alu IS
		PORT (	
					a 		 : IN STD_LOGIC_VECTOR(31 DOWNTO 0);			--1st operand									(32-bit)				
					b 	 	 : IN STD_LOGIC_VECTOR(31 DOWNTO 0);			--2nd operand									(32-bit)
					op 	 : IN STD_LOGIC_VECTOR(2 DOWNTO 0);				--Operation Selection (Control Line)	(3-bit)
					result : INOUT STD_LOGIC_VECTOR(31 DOWNTO 0);		--Operation Result 							(32-bit)
					cout 	 : OUT STD_LOGIC;										--Carry out flag								(1-bit)
					zero   : OUT STD_LOGIC   									--Zero Flag										(1-bit)
			);
	END COMPONENT;
	
	
	------------------------------------------INTERNAL SIGNALS---------------------------------------------------
	SIGNAL data_bus : STD_LOGIC_VECTOR(31 DOWNTO 0);						--output of DATA_MUX
	
	SIGNAL sig_lze, sig_uze : STD_LOGIC_VECTOR(31 DOWNTO 0);				--output of zero extenders
	
	SIGNAL sig_IR, sig_regA, sig_regB : STD_LOGIC_VECTOR(31 DOWNTO 0);--output of registers and IR
	
	SIGNAL sig_muxA, sig_muxB: STD_LOGIC_VECTOR(31 DOWNTO 0);			--output of 2 MUX for registers
	
	SIGNAL signal_ADDR_OUT : STD_LOGIC_VECTOR(31 DOWNTO 0);				--output of PC
	
	SIGNAL sig_mem_IN, sig_mem_OUT : STD_LOGIC_VECTOR(31 DOWNTO 0);	--I/0 of data memory
	
	SIGNAL addr : UNSIGNED(7 DOWNTO 0);											--output of reducer (input address to data_mem)
	
	SIGNAL sig_op1, sig_op2, sig_alu : STD_LOGIC_VECTOR(31 DOWNTO 0);	--output from 2 operand to ALU and ALU itself
	
	SIGNAL sig_cout, sig_zero : STD_LOGIC;										--output of 2 flags from ALU (1-bit)
	
	SIGNAL CCR_signal : STD_LOGIC_VECTOR(7 DOWNTO 0);						--output of Condition Code Register marked by flags (8-bit)
	
	-----------------------------------------PORT MAPPINGS-----------------------------------------------------------
	
	BEGIN
		
		-- IR Port Mapping
		ir:		register32 PORT MAP (
													d		=> data_bus,
													ld 	=> Ld_IR,
													clk 	=> Clk,
													clr 	=> Clr_IR,
													Q 		=> sig_IR
										);
		Out_IR <= sig_IR; -- output of IR is assgined the value of signal coming from IR
		
		-- Upper Zero Extenders
		uze1:		uze	PORT MAP (
											source	=> sig_IR,
											dest		=> sig_uze
								);
		-- Lower Zero Extenders
		lze1:		lze	PORT MAP (										
											source	=> sig_IR,				-- note that for all 4 components of LZE, source is IR
											dest		=> sig_lze				
								);
	   
		-- Program Counter (PC)
		pc_compo:	pc PORT MAP (
											clr	=> Clr_PC,
											clk	=> Clk,
											ld		=>	Ld_PC,
											inc	=> Inc_PC,
											d		=> sig_lze,
											q		=> signal_ADDR_OUT
								);
		Out_PC 	<= signal_ADDR_OUT;	-- output of register
		ADDR_OUT <= signal_ADDR_OUT;	-- used for debugging
		
		-- MUX A
		muxA	: mux_2to1	PORT MAP (
												in0	=> data_bus,
												in1	=> sig_lze,
												sel	=> A_MUX,
												y		=> sig_muxA
									);		
		-- MUX B
		muxB	: mux_2to1	PORT MAP (
												in0	=> data_bus,
												in1	=> sig_lze,
												sel	=> B_MUX,
												y		=> sig_muxB
									);
		-- Register A
		regA	: register32	PORT MAP (
													d		=> sig_muxA,
													ld 	=> Ld_A,
													clk 	=> Clk,
													clr 	=> Clr_A,
													Q 		=> sig_regA
										);
										
		-- Register B
		regB	: register32	PORT MAP (
													d		=> sig_muxB,
													ld 	=> Ld_B,
													clk 	=> Clk,
													clr 	=> Clr_B,
													Q 		=> sig_regB
										);
		Out_A <= sig_regA;					-- Register outputs
		Out_B <= sig_regB;
		
		-- Data Memory
		reducer	: red PORT MAP (
											source	=> sig_IR,
											dest		=>	addr
								);
		mux_Data : mux_2to1 PORT MAP (								--go to data_in of Data Memory
													in0	=> sig_regA,
													in1	=> sig_regB,
													sel	=> Reg_Mux,
													y		=> sig_mem_IN
										);
		dataMem 	: data_mem PORT MAP (
													clk		=> mClk,		--data memory clock is twice as fast a system clock since read and write									
													addr 		=> addr,
													data_in 	=> sig_mem_IN,
													wen 		=> WEN,
													en 		=> EN,
													data_out	=> sig_mem_OUT
										);			
		--For Debugging Purposes								
		MEM_ADDR <= STD_LOGIC_VECTOR(addr); 	-- addr needs to be casted to MEM_ADDR type since was different type prior
		MEM_IN 	<= sig_mem_IN;
		MEM_OUT 	<= sig_mem_OUT;
		
		-- ALU
		operand1 :	mux_2to1 PORT MAP (
													in0	=> sig_regA,
													in1	=> sig_uze,
													sel	=> IM_MUX1,
													y		=> sig_op1
										);
		operand2 :	mux_4to1 PORT MAP (
													in0	=> sig_regB,
													in1	=> sig_lze,
													in2   => x"00000001", -- 'x' represents HEX base. constant 1 (32-bit)
													in3	=> "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ",
													sel	=> IM_MUX2,
													y		=> sig_op2	
										);
		alu_Unit	:	alu PORT MAP (
												a			=> sig_op1, 		 								
												b 	 	 	=>	sig_op2,				
												op 	   => ALU_Op,
												result   =>	sig_alu,
												cout 	   => sig_cout,
												zero     => sig_zero
								);
		regCarry : register1 PORT MAP (
													d		=> sig_cout,
													ld		=> Ld_C,
													clk	=> clk,
													clr	=> Clr_C,
													Q		=> CCR_signal(0)			-- Carry Flag is bit 0 in CCR
										);
		regZero : register1 PORT MAP (
													d		=> sig_zero,
													ld		=> Ld_Z,
													clk	=> clk,
													clr	=> Clr_Z,
													Q		=> CCR_signal(2)			-- Zero Flag is bit 2 in CCR
										);
		Out_C <= CCR_signal(0);			--For Debugging Purposes
		Out_Z <= CCR_signal(2);
		
		-- Operation Selector (data_bus is output of MUX)
		operationSel	: mux_4to1 PORT MAP (
															in0	=> DATA_IN,
															in1	=> sig_mem_OUT,
															in2	=> sig_alu,
															in3	=> "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ",
															sel	=> DATA_MUX,
															y		=> data_bus
												);
		DATA_OUT <= data_bus;					-- Output of Datapath (for debugging purposes)
END description;
-------------------------------------------------------------------------------------------------------------------------